<?php

namespace App\Http\Controllers;

use Laravel\Lumen\Routing\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Lang;
use App\Helpers\AccessLevelHelper;

class PositionsController extends Controller
{
  private $request;
  private $user_permissions;

  public function __construct(Request $request)
  {
    $this->request = $request;
    $this->user_permissions = $this->request->get('user_permissions');
    $this->user_role = $this->request->get('user_role');
  }

  public function getAll()
  {
    return DB::table('position')
      ->join('location', 'position.fk_location', '=', 'location.location_id')
      ->join('job', 'position.fk_job', '=', 'job.job_id')
      ->get();
  }

  public function getAvailableLocations()
  {
    return DB::table('location')->get();
  }

  public function getAvailableJobs()
  {
    return DB::table('job')->get();
  }

  public function createPosition()
  {
    $this->validate($this->request, [
      'position_access_group' => 'required',
      'position_spot_number' => 'required',
      'location_id' => 'required',
      'job_id' => 'required'
    ], [lang::get('validation.required')]);

    $has_permitted_role = AccessLevelHelper::hasPermittedRole($this->user_role, 'responsable');

    $new_position_access_group = $this->request->input('position_access_group');
    $new_position_spot_number = $this->request->input('position_spot_number');
    $new_location_id = $this->request->input('location_id');
    $new_job_id = $this->request->input('job_id');


    if ($has_permitted_role) {
      $inserted_id = DB::table('position')->insertGetId([
        "position_access_group" => $new_position_access_group,
        "position_spot_number" => $new_position_spot_number,
        "fk_location" => $new_location_id,
        "fk_job" => $new_job_id
      ]);
      return ["message" => lang::get('http.success.created.position'), "id" => $inserted_id];
    } else {
      return abort(403, lang::get('http.unauthorized'));
    }
  }

  public function updatePosition($id)
  {
    $this->validate($this->request, [
      'position_access_group' => 'required',
      'position_spot_number' => 'required',
      'location_id' => 'required',
      'job_id' => 'required'
    ], [lang::get('validation.required')]);

    $has_permitted_role = AccessLevelHelper::hasPermittedRole($this->user_role, 'responsable');

    $position_access_group = $this->request->input('position_access_group');
    $position_spot_number = $this->request->input('position_spot_number');
    $location_id = $this->request->input('location_id');
    $job_id = $this->request->input('job_id');

    $wanted_position_exists = DB::table('position')->where('position_id', $id)->exists();

    if ($wanted_position_exists && $has_permitted_role) {
      DB::table('position')->where('position_id', $id)->update([
        "position_access_group" => $position_access_group,
        "position_spot_number" => $position_spot_number,
        "fk_location" => $location_id,
        "fk_job" => $job_id
      ]);
      return ["message" => lang::get('http.success.updated.position'), "id" => $id];
    } else {
      return abort(403, lang::get('http.unauthorized'));
    }
  }

  public function deletePosition($id) {
    $has_permitted_role = AccessLevelHelper::hasPermittedRole($this->user_role, 'responsable');

    $wanted_position_exists = DB::table('position')->where('position_id', $id)->exists();
    $wanted_position_in_use = DB::table('applicant')->where('fk_position', $id)->exists();

    if ($wanted_position_exists && $has_permitted_role) {
      if ($wanted_position_in_use) {
        return abort(403, lang::get('http.error.deleted.position'));
      } else {
        DB::table('position')->where('position_id', $id)->delete();
        return ["message" => lang::get('http.success.deleted.position')];
      }
    } else {
      return abort(403, lang::get('http.unauthorized'));
    }
  }

  public function createLocation () {
    $this->validate($this->request, [
      'location_site' => 'required'
    ], [lang::get('validation.required')]);

    $has_permitted_role = AccessLevelHelper::hasPermittedRole($this->user_role, 'responsable');

    $new_location = $this->request->input('location_site');

    if ($has_permitted_role) {
      $inserted_id = DB::table('location')->insertGetId([
        "location_site" => $new_location
      ]);
      return ["message" => lang::get('http.success.created.location'), "id" => $inserted_id];
    } else {
      return abort(403, lang::get('http.unauthorized'));
    }
  }

  public function createJob () {
    $this->validate($this->request, [
      'job_full_value' => 'required'
    ], [lang::get('validation.required')]);

    $has_permitted_role = AccessLevelHelper::hasPermittedRole($this->user_role, 'responsable');

    $new_value = $this->request->input('job_full_value');
    $new_short_value = preg_replace('/[0-9]+/', '', preg_replace('/[^A-Za-z0-9\-]/', '', preg_replace('/\s+/', '', $new_value)));

    if ($has_permitted_role) {
      $inserted_id = DB::table('job')->insertGetId([
        "job_full_value" => $new_value,
        "job_short_value" => $new_short_value
      ]);
      return ["message" => lang::get('http.success.created.job'), "id" => $inserted_id];
    } else {
      return abort(403, lang::get('http.unauthorized'));
    }
  }
}
